/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Show aspects of 3D graphics programming
 *****************************************************************************/

PImage textureImg;  // texture image

int mode = 0;       // mode index
int numModes = 5;

void setup() {
  size(512, 512, P3D);
  textureImg = loadImage("lena.png");
  
  sphereDetail(10);  // default is 30
}



void draw() {
  background(0);
  
  if (mode == 0) {
    // show wireframes
    camera();
    noLights();
    stroke(255);
    noFill(); 
  }
  else if (mode == 1) {
    // show surfaces and texture with default lighting
    noStroke();
    fill(255);
    lights();  
  }
  else if (mode >= 2) {
    // show surfaces and texture with manual lighting
    ambientLight(0, 0, 128);
    directionalLight(0, 255, 0, -1, 0, 0);
    spotLight(255, 0, 0, width/2, height/2, 400, 0, 0, -1, PI/4, 2);
  }
  
  
  if (mode < 3) {
    // rotate scene according to mouse position
    translate(width/2, height/2, 0);
    rotateY(mouseX/(float)width*2*PI);
    rotateX(mouseY/(float)height*2*PI);
  }
  else if (mode == 3) {
    // translate camera according to mouse position
    camera(mouseX, mouseY, (height/2) / tan(PI/6), mouseX, mouseY, 0, 0, 1, 0);
    translate(width/2, height/2, 0);
  }
  else if (mode == 4) {
    // rotate camera according to mouse position
    beginCamera();
    camera();
    translate(width/2, height/2, 0);
    rotateY(mouseX/(float)width*2*PI);
    rotateX(mouseY/(float)height*2*PI);
    endCamera();
     
  }

  // draw box
  box(100);
  
  // draw sphere 
  pushMatrix();
  translate(0, 100 + 50*cos(millis()/1000.0), 0);
  sphere(50);
  popMatrix(); 
  
  // draw pyramid
  pushMatrix();
  translate(0, -150, 0);
  rotateX(PI/2);
  pyramid(50);
  popMatrix();
}


// define pyramid function
void pyramid(float size) {
  
  textureMode(NORMAL);
  
  // base
  beginShape();
  if (mode != 0)
    texture(textureImg);
  vertex(-size, -size, -size, 0, 0);
  vertex(-size,  size, -size, 0, 1);
  vertex( size,  size, -size, 1, 1);
  vertex( size, -size, -size, 1, 0);
  endShape(CLOSE);
 
  // apsis
  beginShape(TRIANGLE_FAN);
  if (mode != 0)
    texture(textureImg);
  vertex(    0,     0,  size, 0.5, 0);
  vertex(-size, -size, -size, 0, 1);
  vertex(-size,  size, -size, 0.25, 1);
  vertex( size,  size, -size, 0.5, 1);
  vertex( size, -size, -size, 0.75, 1);
  vertex(-size, -size, -size, 1, 1);
  endShape();
}


void mouseClicked() {
  
  if (mouseButton == LEFT) {
    mode = (mode + 1) % numModes;
  }
}
